/*
 *  Get full filename
 *  Copyright (c) 2003 by Jaroslav Kysela <perex@suse.cz>
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation; either version 2.1 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
  
#include <string.h>
#include <errno.h>
#ifdef HAS_WORDEXP
#include <wordexp.h>
#else
#include <glob.h>
#endif
#include <assert.h>

int snd_user_file(const char *file, char **result)
{
#ifdef HAS_WORDEXP
	wordexp_t we;
#else
	glob_t globbuf;	
#endif
	int err;
	
	assert(file && result);
#ifdef HAS_WORDEXP
	err = wordexp(file, &we, WRDE_NOCMD);
	switch (err) {
	case WRDE_NOSPACE:
		return -ENOMEM;
	case 0:
		if (we.we_wordc == 1)
			break;
		/* fall thru */
	default:
		wordfree(&we);
		return -EINVAL;
	}
	*result = strdup(we.we_wordv[0]);
	if (*result == NULL)
		return -ENOMEM;
	wordfree(&we);
#else
	err = glob(file, 0, NULL, &globbuf);
	switch (err) {
	case GLOB_NOSPACE:
		return -ENOMEM;
	case 0:
		if (globbuf.gl_pathc == 1)
			break;
		/* Fall through */
	default:
		globfree(&globbuf);
		return -EINVAL;
	}
	*result = strdup(globbuf.gl_pathv[0]);
	if (*result == NULL)
		return -ENOMEM;
	globfree(&globbuf);
#endif
	return 0;
}
