/* vi: set sw=4 ts=4: */
/*
 * Mini reboot implementation for busybox
 *
 * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/reboot.h>
#include <asm/fcntl.h>
#include <stdio.h>
#include "busybox.h"
#include "init_shared.h"

#undef DO_IOC_REBOOT
#ifdef DO_IOC_REBOOT
#include <linux/ioc.h>
static int ioc_reboot(void)
{
    static int  fd;
    int         rc = 0;

    /* Open the IOC driver and issue a "Reset Fiona" command to do a
     * board hardware reset.
     */
    fd = open(IOC_DRIVER_PATH, O_RDWR);
    if (fd < 0)
    {
        fprintf(stderr, "ERROR: Could not open %s driver.\n",IOC_DRIVER_PATH);
        // Something went wrong, reboot the old fashion way
        goto errExit;
    }
    ioctl(fd, IOC_RESET_FIONA_IOCTL, NULL);

    // We shouldn't get here, but if we do, pause a bit (to make sure prior reboot is not
    // still coming) and then reboot the old fashion way...
    sleep(5);

errExit:
    close(fd);
	return kill_init(SIGTERM);
}
#endif

extern int reboot_main(int argc, char **argv)
{
	char *delay; /* delay in seconds before rebooting */

	if(bb_getopt_ulflags(argc, argv, "d:", &delay)) {
		sleep(atoi(delay));
	}

#ifndef CONFIG_INIT
#ifndef RB_AUTOBOOT
#define RB_AUTOBOOT		0x01234567
#endif
	return(bb_shutdown_system(RB_AUTOBOOT));
#else
#ifdef DO_IOC_REBOOT
    return ioc_reboot();
#else
	return kill_init(SIGTERM);
#endif
#endif
}

/*
Local Variables:
c-file-style: "linux"
c-basic-offset: 4
tab-width: 4
End:
*/
