/* 
 * Globals used to communicate between the bootloader (U-Boot) and the
 * the kernel (Linux) across boot-ups and restarts.
 * 
 * Copyright (C)2006 Lab126, Inc.  All rights reserved.
 */

#include "boot_globals.h"

// warm_restart_flag

void set_warm_restart_flag(unsigned long warm_restart_flag)
{
    get_boot_globals()->globals.warm_restart_flag = warm_restart_flag;
}

unsigned long get_warm_restart_flag(void)
{
    return get_boot_globals()->globals.warm_restart_flag;
}

// apollo_init_display_flag

void set_apollo_init_display_flag(unsigned long apollo_init_display_flag)
{
    get_boot_globals()->globals.apollo_init_display_flag = apollo_init_display_flag;
}

unsigned long get_apollo_init_display_flag(void)
{
    return get_boot_globals()->globals.apollo_init_display_flag;
}

EXPORT_SYMBOL(set_apollo_init_display_flag);
EXPORT_SYMBOL(get_apollo_init_display_flag);

// dirty_boot_flag

void set_dirty_boot_flag(unsigned char dirty_boot_flag)
{
    get_boot_globals()->globals.dirty_boot_flag = dirty_boot_flag;
}

unsigned long get_dirty_boot_flag(void)
{
    return get_boot_globals()->globals.dirty_boot_flag;
}

// progress_bar_value

void set_progress_bar_value(int progress_bar_value)
{
    get_boot_globals()->globals.progress_bar_value = progress_bar_value;
}

int get_progress_bar_value(void)
{
    return get_boot_globals()->globals.progress_bar_value;
}

// user_boot_screen
//

void set_user_boot_screen(int user_boot_screen)
{
    get_boot_globals()->globals.user_boot_screen = user_boot_screen;
}

int get_user_boot_screen(void)
{
    return get_boot_globals()->globals.user_boot_screen;
}

EXPORT_SYMBOL(set_user_boot_screen);
EXPORT_SYMBOL(get_user_boot_screen);

// update_flag

void set_update_flag(unsigned long update_flag)
{
    get_boot_globals()->globals.update_flag = update_flag;
}

unsigned long get_update_flag(void)
{
    return get_boot_globals()->globals.update_flag;
}

// update_data

void set_update_data(unsigned long update_data)
{
    get_boot_globals()->globals.update_data = update_data;
}

unsigned long get_update_data(void)
{
    return get_boot_globals()->globals.update_data;
}

// scratch

void set_scratch(char *scratch)
{
    if ( scratch && (SCRATCH_SIZE >= strlen(scratch)) )
        strcpy(get_boot_globals()->globals.scratch, scratch);
}

char *get_scratch(void)
{
    return get_boot_globals()->globals.scratch;
}

// panel_id

void set_panel_id(panel_id_t *panel_id)
{
    if ( panel_id )
         memcpy(&get_boot_globals()->globals.panel_id, panel_id, sizeof(panel_id_t));
}

panel_id_t *get_panel_id(void)
{
    return &get_boot_globals()->globals.panel_id;
}

EXPORT_SYMBOL(get_panel_id);

void clear_panel_id(void)
{
    memset(&get_boot_globals()->globals.panel_id, 0, sizeof(panel_id_t));
}

// async/sync_settings

void set_async_settings(sync_async_t *async_settings)
{
    if ( async_settings )
        memcpy(&get_boot_globals()->globals.async_settings, async_settings, sizeof(sync_async_t));
}

sync_async_t *get_async_settings(void)
{
    return &get_boot_globals()->globals.async_settings;
}

void set_sync_settings(sync_async_t *sync_settings)
{
    if ( sync_settings )
        memcpy(&get_boot_globals()->globals.sync_settings, sync_settings, sizeof(sync_async_t));
}

sync_async_t *get_sync_settings(void)
{
    return &get_boot_globals()->globals.sync_settings;
}

void clear_async_sync_settings(void)
{
    memset(&get_boot_globals()->globals.async_settings, 0, sizeof(sync_async_t));
    memset(&get_boot_globals()->globals.sync_settings,  0, sizeof(sync_async_t));
}

// clear_dbs

void set_clear_dbs(int clear_dbs)
{
    get_boot_globals()->globals.clear_dbs = clear_dbs;
}

int get_clear_dbs(void)
{
    return get_boot_globals()->globals.clear_dbs;
}

// framebuffer_start

void set_framebuffer_start(unsigned long framebuffer_start)
{
    get_boot_globals()->globals.framebuffer_start = framebuffer_start;
}

unsigned long get_framebuffer_start(void)
{
    return get_boot_globals()->globals.framebuffer_start;
}

EXPORT_SYMBOL(get_framebuffer_start);

// drivemode_screen_ready

void set_drivemode_screen_ready(int drivemode_screen_ready)
{
    get_boot_globals()->globals.drivemode_screen_ready = drivemode_screen_ready;
}

int get_drivemode_screen_ready(void)
{
    return get_boot_globals()->globals.drivemode_screen_ready;
}

EXPORT_SYMBOL(set_drivemode_screen_ready);
EXPORT_SYMBOL(get_drivemode_screen_ready);

// hw_flags

void set_hw_flags(hw_flags_t *hw_flags)
{
    if ( hw_flags )
        memcpy(&get_boot_globals()->globals.hw_flags, hw_flags, sizeof(hw_flags_t));
}

hw_flags_t *get_hw_flags(void)
{
    return &get_boot_globals()->globals.hw_flags;
}

EXPORT_SYMBOL(get_hw_flags);

// framework_started, framework_running

void set_framework_started(int framework_started)
{
    get_boot_globals()->globals.framework_started = framework_started;
}

int get_framework_started(void)
{
    return get_boot_globals()->globals.framework_started;
}

void set_framework_running(int framework_running)
{
    get_boot_globals()->globals.framework_running = framework_running;
}

int get_framework_running(void)
{
    return get_boot_globals()->globals.framework_running;
}

EXPORT_SYMBOL(get_framework_running);

// screen_clear

void set_screen_clear(int screen_clear)
{
    get_boot_globals()->globals.screen_clear = screen_clear;
}

int get_screen_clear(void)
{
    return get_boot_globals()->globals.screen_clear;
}

EXPORT_SYMBOL(set_screen_clear);
EXPORT_SYMBOL(get_screen_clear);

// battery_watermarks

void set_battery_watermarks(bat_level_t *battery_watermarks)
{
    if ( battery_watermarks )
        memcpy(&get_boot_globals()->globals.battery_watermarks, battery_watermarks, sizeof(bat_level_t));
}

bat_level_t *get_battery_watermarks(void)
{
    return &get_boot_globals()->globals.battery_watermarks;
}

// dev_mode

void set_dev_mode(int dev_mode)
{
    get_boot_globals()->globals.dev_mode = dev_mode;
}

int get_dev_mode(void)
{
    return get_boot_globals()->globals.dev_mode;
}

// env_script

void set_env_script(char *env_script)
{
    if ( env_script && (ENV_SCRIPT_SIZE >= strlen(env_script)) )
        strcpy(get_boot_globals()->globals.env_script, env_script);
}

char *get_env_script(void)
{
    return get_boot_globals()->globals.env_script;
}

// fb_ioctl

void set_fb_ioctl(sys_ioctl_t fb_ioctl)
{
    get_boot_globals()->globals.fb_ioctl = fb_ioctl;
}

sys_ioctl_t get_fb_ioctl(void)
{
    return get_boot_globals()->globals.fb_ioctl;
}

EXPORT_SYMBOL(set_fb_ioctl);
EXPORT_SYMBOL(get_fb_ioctl);

// fb_init_flag

void set_fb_init_flag(unsigned long fb_init_flag)
{
    get_boot_globals()->globals.fb_init_flag = fb_init_flag;
}

unsigned long get_fb_init_flag(void)
{
    return get_boot_globals()->globals.fb_init_flag;
}

EXPORT_SYMBOL(set_fb_init_flag);
EXPORT_SYMBOL(get_fb_init_flag);

// op_amp_offset, update_op_amp_offset_env

void set_update_op_amp_offset_env(int update_op_amp_offset_env)
{
    get_boot_globals()->globals.update_op_amp_offset_env = update_op_amp_offset_env;
}

int get_update_op_amp_offset_env(void)
{
    return get_boot_globals()->globals.update_op_amp_offset_env;
}

void set_op_amp_offset(int op_amp_offset)
{
    get_boot_globals()->globals.op_amp_offset = op_amp_offset;
}

int get_op_amp_offset(void)
{
    return get_boot_globals()->globals.op_amp_offset;
}

// rve_value

void set_rve_value(unsigned long rve_value)
{
    get_boot_globals()->globals.rve_value = rve_value;
}

unsigned long get_rve_value(void)
{
    return get_boot_globals()->globals.rve_value;
}

// gain_value

void set_gain_value(unsigned long gain_value)
{
    get_boot_globals()->globals.gain_value = gain_value;
}

unsigned long get_gain_value(void)
{
    return get_boot_globals()->globals.gain_value;
}
