#ifndef __IOC_HAL_H___
#define __IOC_HAL_H___

#include <linux/autoconf.h>
#include <linux/ioc.h>

#undef LOCK_DEBUG
#ifdef LOCK_DEBUG
  #define lockdebug(x...)       printk(x)
#else
  #define lockdebug(x...)       do {} while (0)
#endif


// HAL Layer for IOC Interface to CPU
struct IOC_HAL_Interface {
    int (*IOC_HAL_Send)     (struct IOC_HAL_Interface *hal, char *buf, int size);
    int (*IOC_HAL_DataReady)(struct IOC_HAL_Interface *hal);
    int (*IOC_HAL_Receive)  (struct IOC_HAL_Interface *hal, char *buf, int size);
    int (*IOC_HAL_SendRcv)  (struct IOC_HAL_Interface *hal, char *sBuf, int sSize, char *rBuf, int rSize);
    int (*IOC_HAL_ResetChip)(struct IOC_HAL_Interface *hal);
    struct packet_element * (*IOC_HAL_GetRxPacket)(struct IOC_HAL_Interface *hal);
    struct packet_element * (*IOC_HAL_GetWaitRxPacket)(struct IOC_HAL_Interface *hal);
    int (*IOC_HAL_FlushRxQueue)(struct IOC_HAL_Interface *hal);
    int (*IOC_HAL_FlushTxQueue)(struct IOC_HAL_Interface *hal);
    void *private;  // Private storage for HAL's personal use

};
typedef struct IOC_HAL_Interface IOC_HAL_Interface_T;

struct rx_holding_bin {
    packet_element_t    packet[MAX_RX_PACKETS_TO_BUFFER];
    int                 head;
    int                 tail;
};
typedef struct rx_holding_bin rx_holding_bin_t;


struct rx_queue {
    struct list_head        packet_list;
    spinlock_t              lock;
    wait_queue_head_t       client_list;
    rx_holding_bin_t        rx_holding_bin;
    struct fasync_struct    *fasync;
};
typedef struct rx_queue rx_queue_t;


struct fiona_struct {
    u8 packet[8];
    int pos;
    int length;

    u8 cached_mcr;
    u8 last_msr;
};
typedef struct fiona_struct fiona_packet_t;


// Points to serial port to grab user input from
struct Fiona_IOC_HAL_Globals {
    int                         signature;
    int                         serial_fd;
    int                         serial_inited;
    rx_queue_t                  rx_q;
    spinlock_t                  ioc_lock;
    IOC_HAL_Interface_T         hal;
};
typedef struct Fiona_IOC_HAL_Globals Fiona_IOC_HAL_Globals_T;




// HAL interface routines
IOC_HAL_Interface_T  *IOC_HAL_Init(void);
int IOC_HAL_Destroy(IOC_HAL_Interface_T *hal);

// IOC Keyboard Device register routine
typedef void (kbd_report_key_proc)(int scancode, int pressed);// *kbd_report_key_proc;

int register_ioc_keyboard_handler(kbd_report_key_proc *handler);
void unregister_ioc_keyboard_handler(kbd_report_key_proc *handler);

#endif  // __IOC_HAL_H___
