/********************************************************************
 * Philips ISP176x Host Controller Interface code file
 *
 * (c) 2002 Koninklijke Philips Electronics N.V. All rights reserved. <usb.linux@philips.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name: otg.c
 *
 * Refering linux kernel version 2.6.9
 *
 * History:
 *
 * Date                  Author                  Comments
 * --------------------------------------------------------------------
 * June 21, 2006          krishan                 Initial version
 * 
 **********************************************************************
 */

#ifdef OTG


/*otg device*/
struct usb_device *otgdev = 0;
/*otg hub urb*/
struct urb    *otgurb; 


/*************************otg handling****************************/
#define OTG_DEVICE_ADDRESS  (2)
#define OTG_PORT_NUMBER     (0)



/*return otghub from here*/
struct usb_device *
phci_register_otg_device (struct isp1762_dev *dev)
{
	if(otgdev && 
	   otgdev->devnum == 0x2)
	return otgdev;

  return NULL;	
	
}

/*suspend the otg port(0)
 * needed when port is switching
 * from host to device
 * */
int
phci_suspend_otg_port(struct isp1762_dev *dev,
			u32   command)
{
	int status = 0;
	if ((otgdev == 0) ||
	    (otgdev->devnum != 0x2)){
		err("not a valid/ no otg hub\n");
		err("otg port can not be suspended\n");
		return -ENODEV;
	}

	otgdev->otgstate = USB_OTG_SUSPEND;
	if(otgurb->status == -EINPROGRESS)
		otgurb->status = 0;
	
	/*complete the urb*/
         otgurb->complete(otgurb,NULL);	

	 /*reset the otghub urb status*/
	 otgurb->status = -EINPROGRESS;
	 return status;
}

/*set the flag to enumerate the device*/
int
phci_enumerate_otg_port(struct isp1762_dev *dev,
			u32 command)
{
	/*set the flag to enumerate*/
	/*connect change interrupt will happen from 
	 * phci_intl_worker only
	 * */
	
	otgdev->otgstate = USB_OTG_ENUMERATE;
	
	if(otgurb->status == -EINPROGRESS)
                otgurb->status = 0;

	 /*complete the urb*/
	      otgurb->complete(otgurb,NULL);

	  /*reset the otghub urb status*/
	         otgurb->status = -EINPROGRESS;
			  

	return 0;
}

/*host controller resume sequence at otg port*/
static int
phci_resume_otg_port(struct isp1762_dev *dev,
	              u32   command)
{
        int status = 0;
        struct usb_device *fakedevice;
        //struct usb_port_status portsts;
        //unsigned short portstatus, portchange;
        printk("%s: enter\n",__FUNCTION__);

	if(otgdev == 0){
	        printk("no device\n");
	       return -ENODEV;
	       }
	if(otgdev->devnum != OTG_DEVICE_ADDRESS){
	       err("wrong otg device, can not suspend\n");
	       return -ENODEV;
               }

	fakedevice = kmalloc(sizeof *fakedevice, GFP_KERNEL);
	if(!fakedevice){
		err("can not allocate memory for fake device\n");
		return -ENOMEM;
	}


//	if(usb_hub_port_reset(otgdev,OTG_PORT_NUMBER,fakedevice,10)){
//	 	 info("can not reset the port %d\n", OTG_PORT_NUMBER+1);
//		 return -1;
//		}

	kfree(fakedevice);

//	status = usb_get_port_status(otgdev, OTG_PORT_NUMBER+1,&portsts);
//	if(status < 0){
//		 err("failure to read port status %d\n", status);
//		 return status;
//		}
//
//	portstatus = le16_to_cpu(portsts.wPortStatus);
//	portchange = le16_to_cpu(portsts.wPortChange);
//	info("port status %x, port change %x\n", portstatus, portchange);
/*
	if(portstatus & USB_PORT_STAT_CONNECTION){
		usb_hub_port_connect_change(otgdev,OTG_PORT_NUMBER,&portsts);
	}
*/

       /*resume lets only reset happen*/
       #if 0
       status = usb_control_msg(otgdev, usb_sndctrlpipe(otgdev, 0),
       				USB_REQ_SET_FEATURE, /*request type*/
       				USB_RT_PORT, /*receiver*/
       				USB_PORT_FEAT_RESET, /*feature*/
       				(int)OTG_PORT_NUMBER + 1, /*port number*/
       				NULL, 0, HZ);
       #endif


	// status = usb_control_msg(otgdev, usb_sndctrlpipe(otgdev, 0),
	// 			USB_REQ_SET_FEATURE, USB_RT_PORT,
        // 			USB_PORT_FEAT_POWER,
        // 			OTG_PORT_NUMBER + 1, NULL, 0, HZ);

        printk("%s: exit, status %d\n", __FUNCTION__, status);
	return status;
  }


EXPORT_SYMBOL(phci_register_otg_device);
EXPORT_SYMBOL(phci_enumerate_otg_port);
EXPORT_SYMBOL(phci_suspend_otg_port);
EXPORT_SYMBOL(phci_resume_otg_port);
#endif

