/* 
 * Globals used to communicate between the bootloader (U-Boot) and the
 * the kernel (Linux) across boot-ups and restarts.
 * 
 * Copyright (C)2006-2007 Lab126, Inc.  All rights reserved.
 */

#ifndef _BOOT_GLOBALS_H
#define _BOOT_GLOBALS_H

#ifndef FOR_UBOOT_BUILD //------------- Linux Build

#include <linux/module.h>
#include <linux/string.h>

#include <asm/page.h>
#include <asm/setup.h>

#define FRAMEBUFFER_SIZE            (PAGE_SIZE * 30)
#define ENV_SCRIPT_STR_SZ           256

#else   // ---------------------------- U-Boot Build

#include <common.h>

#define EXPORT_SYMBOL(s)
#define PAGE_SIZE                   (RESERVE_RAM_SIZE)

#endif  // ----------------------------

#define BOOT_GLOBALS_SIZE           (PAGE_SIZE)     // 0x00001000

#define BOOT_GLOBALS_WARM_FLAG      0x6D726177      // 'warm': Warm Boot.
#define BOOT_GLOBALS_COLD_FLAG      0x646C6F63      // 'cold': Cold Boot.

#define BOOT_GLOBALS_UPDATE_FLAG    0x61647075      // 'upda': Firmware Update.
#define BOOT_GLOBALS_REPART_FLAG    0x61706572      // 'repa': Repartion.
#define BOOT_GLOBALS_RESTOR_FLAG    0x74736572      // 'rest': Firmware Restore.
#define BOOT_GLOBALS_SERVIC_FLAG    0x76726573      // 'serv': Service Menu.

#define BOOT_GLOBALS_FB_INIT_FLAG   0x66756266      // 'fbuf': Framebuffer Init'ed.

#define BOOT_GLOBALS_ACK_FLAG       0

#define FREE_SIZE_0                 19              
#define BUILD_REV_SIZE              252
#define SCRATCH_SIZE                256
#define SERIAL_NUM_SIZE             BOARD_SERIALNUM_SIZE
#define ENV_SCRIPT_SIZE             ENV_SCRIPT_STR_SZ

#define NUM_BAT_LEVEL_STAGES        6
#define BAT_LEVEL_UPDT              0
#define BAT_LEVEL_WAN               1
#define BAT_LEVEL_LOW               2
#define BAT_LEVEL_CRIT              3
#define BAT_LEVEL_RSRV1             4
#define BAT_LEVEL_RSRV2             5

#define CLEAR_DBS_UNKNOWN           0               // clear_dbs not set yet
#define CLEAR_DBS_TRUE              1               // for stacked parts
#define CLEAR_DBS_FALSE             2               // for monolithic parts

#define SYNC_SETTINGS_UNKNOWN       0               // sync settings not set
#define SYNC_SETTINGS_INITED        1               // sync settings initialized

#define UNINITIALIZED_RVE_VALUE     0
#define UNINITIALIZED_GAIN_VALUE    0
#define UNINITIALIZED_GAIN_VALUE    0               // could change so make it a symbol
#define UNINITIALIZED_OP_AMP_OFFSET 0xFFFF          // op-amp offset not yet initialized

#define SOFTWARE_UPDATE()           ((BOOT_GLOBALS_UPDATE_FLAG == get_update_flag()) || \
                                     (BOOT_GLOBALS_REPART_FLAG == get_update_flag()) || \
                                     (BOOT_GLOBALS_RESTOR_FLAG == get_update_flag()) || \
                                     (BOOT_GLOBALS_SERVIC_FLAG == get_update_flag()))

#define DISABLE_PDC_SUSPEND()       (1 == get_hw_flags()->disable_pdc_suspend)
#define INVERT_VIDEO()              (1 == get_hw_flags()->invert_video)

#define APOLLO_IS_APOLLO()          (0 == get_panel_id()->apollo_is_pvi)
#define APOLLO_IS_PVI()             (1 == get_panel_id()->apollo_is_pvi)

#define FRAMEWORK_STARTED()         (0 != get_framework_started())
#define FRAMEWORK_RUNNING()         (0 != get_framework_running())

#define IN_DEV_MODE()               (0 != get_dev_mode())

#define FB_INITED()                 (BOOT_GLOBALS_FB_INIT_FLAG == get_fb_init_flag())

struct panel_id_t                                   // 8 bytes
{
    unsigned char   override,
                    platform;
    unsigned short  lot;
    unsigned char   adhesive_run_number,
                    size,
                    apollo_is_pvi,
                    reserved;
};
typedef struct panel_id_t panel_id_t;

struct sync_async_t                                 // 32 bytes
{
    int             initialized;
    unsigned long   onenand_sys_config_1;
    unsigned long   pxa_sxcnfg;
    unsigned long   pxa_mdrefr;
    unsigned long   pxa_msc0;
    unsigned long   reserved[3];
};
typedef struct sync_async_t sync_async_t;

struct hw_flags_t                                   // 4 bytes
{
    unsigned int    disable_pdc_suspend :  1;
    unsigned int    invert_video        :  1;
    unsigned int    reserved            : 30;
};
typedef struct hw_flags_t hw_flags_t;

struct bat_level_t                                  // 16 bytes
{
    int             update_env;
    unsigned short  stage[NUM_BAT_LEVEL_STAGES];
};
typedef struct bat_level_t bat_level_t;

typedef int (*sys_ioctl_t)(unsigned int cmd, unsigned long arg); 

struct bg_in_use_t
{
    unsigned long warm_restart_flag;                // 0xA3FFF000
    unsigned long apollo_init_display_flag;         // 0xA3FFF004
    int           progress_bar_value;               // 0xA3FFF008
    unsigned long bootloader_status;                // 0xA3FFF00C
    unsigned long bootloader_messsage;              // 0xA3FFF010
    unsigned long bootblock_version;                // 0xA3FFF014
    char          free0[FREE_SIZE_0];               // 0xA3FFF018
    unsigned char dirty_boot_flag;                  // 0xA3FFF02B
    int           user_boot_screen;                 // 0xA3FFF02C
    int           boot_ioc_key;                     // 0xA3FFF030
    int           boot_key;                         // 0xA3FFF034
    int           boot_recover;                     // 0xA3FFF038
    unsigned long update_flag;                      // 0xA3FFF03C
    unsigned long update_data;                      // 0xA3FFF040
    unsigned long saved_rtc;                        // 0xA3FFF044
    unsigned long saved_rtc_checksum;               // 0xA3FFF048
    char          scratch[SCRATCH_SIZE];            // 0xA3FFF04C
    panel_id_t    panel_id;                         // 0xA3FFF14C
    int           framework_started;                // 0xA3FFF154
    char          serial_num[SERIAL_NUM_SIZE];      // 0xA3FFF158 (32 bytes)
    sync_async_t  async_settings;                   // 0xA3FFF178
    sync_async_t  sync_settings;                    // 0xA3FFF198
    int           clear_dbs;                        // 0xA3FFF1B8
    unsigned long framebuffer_start;                // 0xA3FFF1BC
    int           drivemode_screen_ready;           // 0xA3FFF1C0
    hw_flags_t    hw_flags;                         // 0xA3FFF1C4
    int           framework_running;                // 0xA3FFF1C8
    int           screen_clear;                     // 0xA3FFF1CC
    bat_level_t   battery_watermarks;               // 0xA3FFF1D0
    int           dev_mode;                         // 0xA3FFF1E0
    int           update_op_amp_offset_env;         // 0xA3FFF1E4
    char          env_script[ENV_SCRIPT_SIZE];      // 0xA3FFF1E8
    sys_ioctl_t   fb_ioctl;                         // 0xA3FFF2E8
    unsigned long fb_init_flag;                     // 0xA3FFF2EC
    char          build_rev[BUILD_REV_SIZE];        // 0xA3FFF2F0
    int           op_amp_offset;                    // 0xA3FFF3EC
    unsigned long rve_value;                        // 0xA3FFF3F0
    unsigned long gain_value;                       // 0xA3FFF3F4
};
typedef struct bg_in_use_t bg_in_use_t;

struct boot_globals_t
{
    bg_in_use_t   globals;
    unsigned char reserved[BOOT_GLOBALS_SIZE - sizeof(bg_in_use_t)];
};
typedef struct boot_globals_t boot_globals_t;

extern boot_globals_t *get_boot_globals(void);

// warm_restart_flag

extern void set_warm_restart_flag(unsigned long warm_restart_flag);
extern unsigned long get_warm_restart_flag(void);

// apollo_init_display_flag

extern void set_apollo_init_display_flag(unsigned long apollo_init_display_flag);
extern unsigned long get_apollo_init_display_flag(void);

// dirty_boot_flag

extern void set_dirty_boot_flag(unsigned char dirty_boot_flag);
extern unsigned long get_dirty_boot_flag(void);

// user_boot_screen

extern void set_user_boot_screen(int user_boot_screen);
extern int get_user_boot_screen(void);

// progress_bar_value

extern void set_progress_bar_value(int progress_bar_value);
extern int get_progress_bar_value(void);

// update_flag

extern void set_update_flag(unsigned long update_flag);
extern unsigned long get_update_flag(void);

// update_data

extern void set_update_data(unsigned long update_data);
extern unsigned long get_update_data(void);

// scratch

extern void set_scratch(char *scratch);
extern char *get_scratch(void);

// panel_id

extern void set_panel_id(panel_id_t *panel_id_t);
extern panel_id_t *get_panel_id(void);

extern void clear_panel_id(void);

// async/sync_settings

extern void set_async_settings(sync_async_t *async_settings);
extern sync_async_t *get_async_settings(void);

extern void set_sync_settings(sync_async_t *sync_settings);
extern sync_async_t *get_sync_settings(void);

extern void clear_async_sync_settings(void);

// clear_dbs

extern void set_clear_dbs(int clear_dbs);
extern int get_clear_dbs(void);

// framebuffer_start

extern void set_framebuffer_start(unsigned long framebuffer_start);
extern unsigned long get_framebuffer_start(void);

// drivemode_screen_ready

extern void set_drivemode_screen_ready(int drivemode_screen_ready);
extern int get_drivemode_screen_ready(void);

// hw_flags

extern void set_hw_flags(hw_flags_t *hw_flags);
extern hw_flags_t *get_hw_flags(void);

// framework_started, framework_running

extern void set_framework_started(int framework_started);
extern int get_framework_started(void);

extern void set_framework_running(int framework_running);
extern int get_framework_running(void);

// screen_clear

extern void set_screen_clear(int screen_clear);
extern int get_screen_clear(void);

// battery_watermarks

extern void set_battery_watermarks(bat_level_t *battery_watermarks);
extern bat_level_t *get_battery_watermarks(void);

// dev_mode

extern void set_dev_mode(int dev_mode);
extern int get_dev_mode(void);

// env_script

extern void set_env_script(char *env_script);
extern char *get_env_script(void);

// fb_ioctl

extern void set_fb_ioctl(sys_ioctl_t fb_ioctl);
extern sys_ioctl_t get_fb_ioctl(void);

// fb_init_flag

extern void set_fb_init_flag(unsigned long fb_init_flag);
extern unsigned long get_fb_init_flag(void);

// op_amp_offset, update_op_amp_offset_env

extern void set_update_op_amp_offset_env(int update_op_amp_offset_env);
extern int get_update_op_amp_offset_env(void);

extern void set_op_amp_offset(int op_amp_offset);
extern int get_op_amp_offset(void);

// rve_value

extern void set_rve_value(unsigned long rve_value);
extern unsigned long get_rve_value(void);

// gain_value

extern void set_gain_value(unsigned long gain_value);
extern unsigned long get_gain_value(void);

#endif  // _BOOT_GLOBALS_H
