#ifndef _EINKFB_H
#define _EINKFB_H

enum splash_screen_type
{
    // Simple (non-composite) splash screens.
    //
    splash_screen_powering_off = 0,
    splash_screen_powering_on,
    
    splash_screen_powering_off_wireless,
    splash_screen_powering_on_wireless,
    
    splash_screen_exit,
    splash_screen_logo,
    
    splash_screen_usb_internal,
    splash_screen_usb_external,
    splash_screen_usb,
    
    splash_screen_sleep,
    splash_screen_update,
    
    num_splash_screens,

    // Composite splash screens & messages.
    //
    splash_screen_drivemode_0,              // Draw no labels.
    splash_screen_drivemode_1,              // Draw only internal label.
    splash_screen_drivemode_2,              // Draw only external label.
    splash_screen_drivemode_3,              // Draw both internal and external labels.
    
    splash_screen_power_off_clear_screen,   // Message: clear screen and power down controller.
    
    splash_screen_sleep_fade,               // Screen dimmed and ALT+FONT displayed.
    splash_screen_fade,                     // Screen dimmed only.
    splash_screen_exit_fade,                // Exit for screen-dimmed (including splash_screen_store_exit) cases.
    
    splash_screen_store_exit,               // Screen dimmed and Amazon "swoosh" displayed.
    
    splash_screen_blank_status_lo,          // Messages for screen dimming.
    splash_screen_blank_status_stub_lo,     //
    splash_screen_blank_status_hi,          //
    splash_screen_blank_status_stub_hi,     //
    splash_screen_sleep_status_lo,          //
    splash_screen_sleep_status_hi,          //
    
    splash_screen_blank_status,             // No dimming, just blank the status area.
    splash_screen_framework_reset,          // No screen activity, just reset all timers, threads, etc....

    splash_screen_screen_saver_picture,     // Screen saver picture and ALT+FONT displayed.
    
    splash_screen_invalid = -1
};
typedef enum splash_screen_type splash_screen_type;

struct fpow_override_t
{
    u_int   cmd;
    u_long  arg;
};
typedef struct fpow_override_t fpow_override_t;

enum fx_type
{
    fx_lighten_to_dark_gray_max = 0,        // Black -> Dark Gray, Dark Gray -> Lite Gray.
    fx_lighten_to_lite_gray_max,            // Black -> Lite Gray, Dark Gray -> White.
    fx_lighten_to_white,                    // All -> White.

    fx_darken_to_lite_gray_min,             // White -> Lite Gray, Lite Gray -> Dark Gray.
    fx_darken_to_dark_gray_min,             // White -> Dark Gray, Lite Gray -> Black.
    fx_darken_to_black,                     // All -> Black.
    
    fx_invert,                              // Black <-> White, Dark Gray <-> Lite Gray.
    
    fx_stipple_lighten_dark,                // Even rows, inject White; odd rows, inject Black.
    fx_stipple_lighten_lite,                // Even rows, inject White; odd rows, inject White (stipple between odd/even rows).
    
    fx_stipple_darken_lite,                 // Even rows, inject Black; odd rows, inject White.
    fx_stipple_darken_dark,                 // Even rows, inject Black; odd rows, inject Black (stipple between odd/even rows).
    
    fx_mask,                                // Only for use with update_partial_t's non-NULL buffer which_fx.
    
    fx_stipple_dark_gray,                   // Inject stipple with Dark Gray, eliminate Black.
    fx_stipple_lite_gray,                   // Inject stipple with Lite Gray, Dark Gray max.
    
    fx_none = -1
};
typedef enum fx_type fx_type;

struct fx_t
{
    int         update_type;                // EINK_FULL_UPDATE_OFF | EINK_FULL_UPDATE_ON
    fx_type     which_fx;
    
    int         exclude_rect,               // EINK_FX_DONT_EXCLUDE | EINK_FX_EXCLUDE_RECT
                x1, y1,                     // Top-left...
                x2, y2;                     // ...bottom-right.
};
typedef struct fx_t fx_t;

struct update_partial_t
{
    int         x1, y1,                     // Top-left...
                x2, y2;                     // ...bottom-right.
    
    fx_type     which_fx;                   // FX, if any, to apply.
        
    u8          *buffer;                    // If NULL, extract from framebuffer, top-left to bottom-right, by rowbytes.
};
typedef struct update_partial_t update_partial_t;

enum screen_saver_t
{
    screen_saver_invalid = 0,
    screen_saver_valid
};
typedef enum screen_saver_t screen_saver_t;

#define EINK_SCREEN_SAVER_BUFFER_VALID      "/sys/devices/platform/eink_fb0/valid_screen_saver_buf"
#define EINK_SCREEN_SAVER_VALID_MSG         "1"
#define EINK_SCREEN_SAVER_INVALID           "0"

#define EINK_SCREEN_SAVER_VALID_MSG_LEN     1

#define EINK_SCREEN_SAVER_BUFFER            "/proc/eink_fb/screen_saver"
#define EINK_FRAME_BUFFER                   "/dev/fb/0"

#define EINK_FULL_UPDATE_OFF                0
#define EINK_FULL_UPDATE_ON                 1

#define EINK_FX_DONT_EXCLUDE                0
#define EINK_FX_EXCLUDE_RECT                1

#define EINK_AUTO_POWER_OFF                 0
#define EINK_AUTO_POWER_ON                  1

#define FBIO_MIN_SCREEN                     splash_screen_powering_off
#define FBIO_MAX_SCREEN                     num_splash_screens
#define FBIO_SCREEN_IN_RANGE(s)             \
    ((FBIO_MIN_SCREEN <= (s)) && (FBIO_MAX_SCREEN > (s)))

#define FBIO_MAGIC_NUMBER                   'F'
#define FBIO_EINK_UPDATE_DISPLAY_ASYNC      _IO(FBIO_MAGIC_NUMBER, 0xde) // 0x46de (EINK_FULL_UPDATE_OFF | EINK_FULL_UPDATE_ON)
#define FBIO_EINK_UPDATE_DISPLAY_SYNC       _IO(FBIO_MAGIC_NUMBER, 0xdb) // 0x46db (EINK_FULL_UPDATE_OFF | EINK_FULL_UPDATE_ON)
#define FBIO_EINK_UPDATE_PARTIAL            _IO(FBIO_MAGIC_NUMBER, 0xdd) // 0x46dd (*update_partial_t)
#define FBIO_EINK_UPDATE_DISPLAY_FX         _IO(FBIO_MAGIC_NUMBER, 0xe4) // 0x46e4 (*fx_t)

#define FBIO_EINK_SPLASH_SCREEN             _IO(FBIO_MAGIC_NUMBER, 0xdc) // 0x46dc (splash_screen_type)
#define FBIO_EINK_SPLASH_SCREEN_SLEEP       _IO(FBIO_MAGIC_NUMBER, 0xe0) // 0x46e0 (splash_screen_type)

#define FBIO_EINK_OFF_CLEAR_SCREEN          _IO(FBIO_MAGIC_NUMBER, 0xdf) // 0x46df (no args)
#define FBIO_EINK_CLEAR_SCREEN              _IO(FBIO_MAGIC_NUMBER, 0xe1) // 0x46e1 (no args)

#define FBIO_EINK_RESTORE_SCREEN            _IO(FBIO_MAGIC_NUMBER, 0xe2) // 0x46e2 (no args)
#define FBIO_EINK_FPOW_OVERRIDE             _IO(FBIO_MAGIC_NUMBER, 0xe3) // 0x46e3 (*fpow_override_t)

#define FBIO_EINK_SYNC_BUFFERS              _IO(FBIO_MAGIC_NUMBER, 0xe5) // 0x46e5 (no args)
#define FBIO_EINK_SET_AUTO_POWER            _IO(FBIO_MAGIC_NUMBER, 0xe6) // 0x46e6 (EINK_AUTO_POWER_OFF | EINK_AUTO_POWER_ON)
#define FBIO_EINK_GET_AUTO_POWER            _IO(FBIO_MAGIC_NUMBER, 0xe7) // 0x46e7 (*int)

#define PROC_EINK_UPDATE_DISPLAY_CLS        0   // FBIO_EINK_CLEAR_SCREEN
#define PROC_EINK_UPDATE_DISPLAY_ASIS       1   // FBIO_EINK_UPDATE_DISPLAY(EINK_FULL_UPDATE_OFF)
#define PROC_EINK_UPDATE_DISPLAY_FULL       2   // FBIO_EINK_UPDATE_DISPLAY(EINK_FULL_UPDATE_ON)
#define PROC_EINK_UPDATE_DISPLAY_PART       3   // FBIO_EINK_UPDATE_PARTIAL
#define PROC_EINK_UPDATE_DISPLAY_REST       4   // FBIO_EINK_RESTORE_SCREEN
#define PROC_EINK_UPDATE_DISPLAY_SCRN       5   // FBIO_EINK_SPLASH_SCREEN
#define PROC_EINK_UPDATE_DISPLAY_OVRD       6   // FBIO_EINK_FPOW_OVERRIDE
#define PROC_EINK_UPDATE_DISPLAY_FX         7   // FBIO_EINK_UPDATE_DISPLAY_FX
#define PROC_EINK_UPDATE_DISPLAY_SYNC       8   // FBIO_EINK_SYNC_BUFFERS

// Inter-module/inter-driver eink ioctl access.
//
extern int fiona_eink_ioctl_stub(unsigned int cmd, unsigned long arg);

#define eink_sys_ioctl(cmd, arg)            (get_fb_ioctl() ? (*get_fb_ioctl())((unsigned int)cmd, (unsigned long)arg)      \
                                                            : fiona_eink_ioctl_stub((unsigned int)cmd, (unsigned long)arg))

#endif // _EINKFB_H
