/*
 * (C) Copyright 2003
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *   patched Sept 2004 David Farrell davidjf2001@yahoo.com djf
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <mmc.h>
#include <asm/errno.h>
#include <asm/arch/hardware.h>

#include <part.h>

//	#define	MMC_DEBUG
//	#define debug	printf

#ifdef CONFIG_MMC

unsigned int get_fiona_board_revision(void);
void mmc_print_stat(void);

void
mmc_print_stat(void)
{

	int j,k=0;
	short stat = MMC_STAT & MMC_STAT_ERRORS;
	if	(stat != 0) printf(""); else
	for (j=1;j<256;j*=2)	{
		switch (stat & j)	{
			case (0x0001UL << 13):	printf("END_CMD_RES. ");
									k++;
									break;
			case (0x0001UL << 12):	printf("PRG_DONE. ");
									k++;
									break;
			case (0x0001UL << 11):	printf("DATA_TRAN_DONE. ");
									k++;
									break;
			case (0x0001UL << 8):	printf("CLK_EN. ");
									k++;
									break;
			case (0x0001UL << 7):	printf("RECV_FIFO_FULL. ");
									k++;
									break;
			case (0x0001UL << 6):	printf("XMIT_FIFO_EMPTY. ");
									k++;
									break;
			case (0x0001UL << 5):	printf("RES_CRC_ERROR. ");
									k++;
									break;
			case (0x0001UL << 4):	printf("SPI_READ_ERROR_TOKEN. ");
									k++;
									break;
			case (0x0001UL << 3):	printf("CRC_READ_ERROR. ");
									k++;
									break;
			case (0x0001UL << 2):	printf("CRC_WRITE_ERROR. ");
									k++;
									break;
			case (0x0001UL << 1):	printf("TIME_OUT_RESPONSE. ");
									k++;
									break;
			case (0x0001UL):		printf("READ_TIME_OUT. ");
									k++;
									break;
			default:				break;
		}
	}
}

extern int
fat_register_device(block_dev_desc_t *dev_desc, int part_no);

static block_dev_desc_t mmc_dev;

block_dev_desc_t * mmc_get_dev(int dev)
{
	return ((block_dev_desc_t *)&mmc_dev);
}

static uchar mmc_buf[MMC_BLOCK_SIZE];
/* static mmc_csd_t mmc_csd; */
static int mmc_ready = 0;

int mmc_block_size = MMC_BLOCK_SIZE;

#define PXA_MMC_NOB_MAX		((1 << 16) - 2)			/* maximum number of blocks */

#define ENV_MBREAD_FLAG		"enable_mbread"			/* mbread environment control flag */
static int mmc_use_mbread = 0;						/* default is multi-block reads disabled */

#define MID_TOSHIBA         0x02

static enum {
	CARD_MMC	= 0,
	CARD_SD_1,
	CARD_SD_2
} card_type = CARD_MMC;


static uchar *
/****************************************************/
mmc_cmd(ushort cmd, ushort argh, ushort argl, ushort cmdat)
/****************************************************/
{
	static uchar resp[20];
	ulong status;
	int words, i;

	MMC_STRPCL = MMC_STRPCL_STOP_CLK;
	MMC_I_MASK = ~MMC_I_MASK_CLK_IS_OFF;
	while (!(MMC_I_REG & MMC_I_REG_CLK_IS_OFF));
	MMC_CMD    = cmd;
	MMC_ARGH   = argh;
	MMC_ARGL   = argl;
	MMC_CMDAT  = cmdat;
	MMC_I_MASK = ~MMC_I_MASK_END_CMD_RES;
	MMC_STRPCL = MMC_STRPCL_START_CLK;
	while (!(MMC_I_REG & MMC_I_REG_END_CMD_RES));

	status = MMC_STAT;
//	printf("\n      status> ");
//	mmc_print_stat();

	if (status & MMC_STAT_TIME_OUT_RESPONSE)
	{
		return 0;
	}

	switch (cmdat & 0x3)
	{
		case MMC_CMDAT_R1:
		case MMC_CMDAT_R3:
			words = 3;
			break;

		case MMC_CMDAT_R2:
			words = 8;
			break;

		default:
			return 0;
	}
	for (i = words-1; i >= 0; i--)
	{
		ulong res_fifo = MMC_RES;
		int offset = i << 1;

		resp[offset] = ((uchar *)&res_fifo)[0];
		resp[offset+1] = ((uchar *)&res_fifo)[1];
	}
#ifdef MMC_DEBUG
	printf("\n     results> ");
	for (i=0; i<words*2; i += 2)
	{
		printf(" %02x", resp[i]);
		printf(" %02x", resp[i+1]);
	}
	printf("\n");
#endif
//	printf("  End CMD%d status:",cmd);
//	mmc_print_stat();
//	printf("\n");
	return resp;
}

int
/****************************************************/
mmc_block_read(uchar *dst, ulong src, ulong len, ulong nrblocks)
/****************************************************/
{
	uchar *resp;
	ushort argh, argl;
	ulong status;
	uchar *origdst;

	origdst	= dst;

	if (len == 0)	return 0;

	debug("Start block_read: \n");
	debug("mmc_block_rd> dst 0x%08x src 0x%08x len %d\n", (ulong)dst, src, len);

	/* the PXA has a limit on the number of blocks that can be transferred
	   (this size is 33MB, which is more than we should ever need to read in
	   this bootloader... ((65536 - 2) * 512) bytes) */
    if (nrblocks >= PXA_MMC_NOB_MAX)
    {
    	printf("BUG: mmc_block_read requested size too large (%d blocks)\nread failed\n", nrblocks);
    	return -1;
    }

	if (card_type != CARD_SD_2) {
		argh = len >> 16;
		argl = len & 0xffff;

		/* set block len */
		resp = mmc_cmd(MMC_CMD_SET_BLOCKLEN, argh, argl, MMC_CMDAT_R1);
	}

	/* send read command */
//	printf("\nBLK READ> %d:",cmd);
//	debug("\n  block_read> 0x%02x 0x%02x 0x%02x 0x%02x", cmd, argh, argl, cmdat);
//	resp = mmc_cmd(MMC_CMD_READ_BLOCK, argh, argl, MMC_CMDAT_R1|MMC_CMDAT_READ|MMC_CMDAT_BLOCK|MMC_CMDAT_DATA_EN);

	MMC_STRPCL = MMC_STRPCL_STOP_CLK;
	MMC_I_MASK = ~MMC_I_MASK_CLK_IS_OFF;
	while (!(MMC_I_REG & MMC_I_REG_CLK_IS_OFF));

	if (mmc_use_mbread) {
		MMC_NOB = nrblocks;
		MMC_CMD = nrblocks > 1 ? MMC_CMD_RD_BLK_MULTI : MMC_CMD_READ_BLOCK;
	}
	else
	{
		MMC_NOB = 1;
		MMC_CMD = MMC_CMD_READ_BLOCK;
	}

	if (card_type == CARD_SD_2) {
		src >>= 9;
	}

	MMC_RDTO = 0xffff;
	MMC_BLKLEN = len;
	MMC_ARGH   = src >> 16;
	MMC_ARGL   = src & 0xffff;
	MMC_CMDAT  = MMC_CMDAT_R1|MMC_CMDAT_READ|MMC_CMDAT_BLOCK|MMC_CMDAT_DATA_EN;
	MMC_I_MASK = ~MMC_I_MASK_END_CMD_RES;
	MMC_STRPCL = MMC_STRPCL_START_CLK;
	while (!(MMC_I_REG & MMC_I_REG_END_CMD_RES));

	status = MMC_STAT;
//	printf("                ");
//	printf("\n      status> ");
	mmc_print_stat();

	if (status & MMC_STAT_TIME_OUT_RESPONSE)
	{
		return 0;
	}

	MMC_I_MASK = ~MMC_I_MASK_RXFIFO_RD_REQ;

    if (mmc_use_mbread || card_type == CARD_SD_2)
    {
		int savelen = len;

		while (nrblocks--)
		{
			len = savelen;

		    while (len)
			{
                if (MMC_I_REG & MMC_I_REG_RXFIFO_RD_REQ) {
    	            *dst++ = MMC_RXFIFO;
        	        --len;
                }
	        }

            status = MMC_STAT;
            if (status & MMC_STAT_ERRORS)
            {
                printf("mmc_block_rd>   MMC_STAT error 0x%08x\n", status & MMC_STAT_ERRORS );
                return -1;
            }
		}
    }
    else
    {
    	int c = 1;

    	while (len)
    	{
    		c--;
    
    		if (c==0)
    		{
    			while (c!=31)
    			{
    				if (MMC_I_REG & MMC_I_REG_RXFIFO_RD_REQ)	c=31;
    			}
    		}
    
    		{
    			*dst++ = MMC_RXFIFO;
    			len--;
    		}
    
    		status = MMC_STAT;
    		if (status & MMC_STAT_ERRORS  )
    		{
    			printf("mmc_block_rd>	MMC_STAT error 0x%08x\n", status & MMC_STAT_ERRORS );
    			return -1;
    		}
    	}
	}

	debug("\n");
	MMC_I_MASK = ~MMC_I_MASK_DATA_TRAN_DONE;
	while (!(MMC_I_REG & MMC_I_REG_DATA_TRAN_DONE));
	status = MMC_STAT;
	if (status & MMC_STAT_ERRORS  )
	{
		printf("mmc_block_rd> MMC_STAT error 0x%08x\n", status & MMC_STAT_ERRORS);
		return -1;
	}
	debug("  End block_read: ");
	mmc_print_stat();
	debug("\n");

	return 0;
}

int
/****************************************************/
mmc_block_write(ulong dst, uchar *src, int len)
/****************************************************/
{
	uchar *resp;
	ushort argh, argl;
	ulong status;

	if (len == 0)
	{
		return 0;
	}

	debug("mmc_block_wr> dst 0x%08x src 0x%08x len %d\n", dst, (ulong)src, len);

	argh = len >> 16;
	argl = len & 0xffff;

	/* set block len */
	resp = mmc_cmd(MMC_CMD_SET_BLOCKLEN, argh, argl, MMC_CMDAT_R1);

	/* send write command */
	argh = dst >> 16;
	argl = dst & 0xffff;
	MMC_STRPCL = MMC_STRPCL_STOP_CLK;
	MMC_NOB = 1;
	MMC_BLKLEN = len;
	resp = mmc_cmd(MMC_CMD_WRITE_BLOCK, argh, argl,
			MMC_CMDAT_R1|MMC_CMDAT_WRITE|MMC_CMDAT_BLOCK|MMC_CMDAT_DATA_EN);

	MMC_I_MASK = ~MMC_I_MASK_TXFIFO_WR_REQ;
	while (len)
	{
		if (MMC_I_REG & MMC_I_REG_TXFIFO_WR_REQ)
		{
			int i, bytes = min(32,len);

			for (i=0; i<bytes; i++)
			{
				MMC_TXFIFO = *src++;
			}
			if (bytes < 32)
			{
				MMC_PRTBUF = MMC_PRTBUF_BUF_PART_FULL;
			}
			len -= bytes;
		}
		status = MMC_STAT;
		if (status & MMC_STAT_ERRORS)
		{
			printf("MMC_STAT error 0x%08x\n", status);
			return -1;
		}
	}
	MMC_I_MASK = ~MMC_I_MASK_DATA_TRAN_DONE;
	while (!(MMC_I_REG & MMC_I_REG_DATA_TRAN_DONE));
	MMC_I_MASK = ~MMC_I_MASK_PRG_DONE;
	while (!(MMC_I_REG & MMC_I_REG_PRG_DONE));
	status = MMC_STAT;
	if (status & MMC_STAT_ERRORS)
	{
		printf("MMC_STAT error 0x%08x\n", status);
		return -1;
	}
		printf("MMC_STAT error 0x%08x\n", status);
	return 0;
}


int
/****************************************************/
mmc_read(ulong src, uchar *dst, int size)
/****************************************************/
{
	ulong end, part_start, part_end, part_len, aligned_start, aligned_end;
	ulong mmc_block_size, mmc_block_address;

	if (size == 0)
	{
		return 0;
	}

	if (!mmc_ready)
	{
		printf("Please initialize the MMC first\n");
		return -1;
	}

	debug("    mmc_read>  MMC is ready.\n");

	mmc_block_size = MMC_BLOCK_SIZE;
	mmc_block_address = ~(mmc_block_size - 1);

	src -= CFG_MMC_BASE;
	end = src + size;
	part_start = ~mmc_block_address & src;
	part_end = ~mmc_block_address & end;
	aligned_start = mmc_block_address & src;
	aligned_end = mmc_block_address & end;

	/* all block aligned accesses */
	debug("    mmc_read>   src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);

	if (part_start)
	{
		part_len = mmc_block_size - part_start;
		debug("    mmc_read>   ps src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
		src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);

		if ((mmc_block_read(mmc_buf, aligned_start, mmc_block_size, 1)) < 0)
		{
			return -1;
		}
		memcpy(dst, mmc_buf+part_start, part_len);
		dst += part_len;
		src += part_len;
	}
	debug("    mmc_read>   src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
	if (mmc_use_mbread)
	{
		if ((mmc_block_read((uchar *)(dst), src, mmc_block_size, (aligned_end - src) / mmc_block_size)) < 0)
		{
			return -1;
		}
	}
	else
	{
    	for (; src < aligned_end; src += mmc_block_size, dst += mmc_block_size)
    	{
    		debug("    mmc_read>   al src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
    		src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);

    		if ((mmc_block_read((uchar *)(dst), src, mmc_block_size, 1)) < 0)
    		{
    			return -1;
    		}
    	}
	}
	debug("    mmc_read>   src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
	if (part_end && src < end)
	{
		debug("    mmc_read>   pe src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
		src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);

		if ((mmc_block_read(mmc_buf, aligned_end, mmc_block_size, 1)) < 0)
		{
			return -1;
		}
		memcpy(dst, mmc_buf, part_end);
	}
	return 0;
}

int
/****************************************************/
mmc_write(uchar *src, ulong dst, int size)
/****************************************************/
{
	ulong end, part_start, part_end, part_len, aligned_start, aligned_end;
	ulong mmc_block_size, mmc_block_address;

	if (size == 0)
	{
		return 0;
	}

	if (!mmc_ready)
	{
		printf("Please initial the MMC first\n");
		return -1;
	}

	debug("   mmc_write>   MMC is ready.\n");

	mmc_block_size = MMC_BLOCK_SIZE;
	mmc_block_address = ~(mmc_block_size - 1);

	dst -= CFG_MMC_BASE;
	end = dst + size;
	part_start = ~mmc_block_address & dst;
	part_end = ~mmc_block_address & end;
	aligned_start = mmc_block_address & dst;
	aligned_end = mmc_block_address & end;

	/* all block aligned accesses */
	debug("src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
	if (part_start)
	{
		part_len = mmc_block_size - part_start;
		debug("   mmc_write>   ps: src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
		(ulong)src, dst, end, part_start, part_end, aligned_start, aligned_end);
		if ((mmc_block_read(mmc_buf, aligned_start, mmc_block_size, 1)) < 0)
		{
			return -1;
		}
		memcpy(mmc_buf+part_start, src, part_len);
		if ((mmc_block_write(aligned_start, mmc_buf, mmc_block_size)) < 0)
		{
			return -1;
		}
		dst += part_len;
		src += part_len;
	}
	debug("   mmc_write>  src: 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
	for (; dst < aligned_end; src += mmc_block_size, dst += mmc_block_size)
	{
		debug("   mmc_write>   al: src 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
		src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
		if ((mmc_block_write(dst, (uchar *)src, mmc_block_size)) < 0)
		{
			return -1;
		}
	}
	debug("   mmc_write>  src: 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
	src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
	if (part_end && dst < end)
	{
		debug("   mmc_write>pe src: 0x%08x dst 0x%08x end 0x%08x pstart 0x%08x pend 0x%08x astart 0x%08x aend 0x%08x\n",
		src, (ulong)dst, end, part_start, part_end, aligned_start, aligned_end);
		if ((mmc_block_read(mmc_buf, aligned_end, mmc_block_size, 1)) < 0)
		{
			return -1;
		}
		memcpy(mmc_buf, src, part_end);
		if ((mmc_block_write(aligned_end, mmc_buf, mmc_block_size)) < 0)
		{
			return -1;
		}
	}
	return 0;
}

static int current_state;	/* djf */
static u16 rca_addr;		/* djf */
#define MMC_ST_IDLE 0
#define MMC_ST_READY 1
#define MMC_ST_IDENT 2
#define MMC_ST_STBY 3
#define MMC_ST_TRAN 4

int mmc_block_size;

ulong
/****************************************************/
mmc_bread(int dev_num, ulong blknr, ulong blkcnt, ulong *dst)
/****************************************************/
{
uchar *resp;

	debug("mmc_bread %08X, %08X, %08X\n",blknr,blkcnt,dst);

	ulong src = blknr * mmc_block_size + CFG_MMC_BASE;

        /* select the card */
        resp = mmc_cmd(7,rca_addr, 0, MMC_CMDAT_R1|MMC_CMDAT_BUSY);
        if(resp)current_state = MMC_ST_TRAN;
	
	mmc_read(src, (uchar *)dst, blkcnt*mmc_block_size);

        /* deselect the card */
        resp = mmc_cmd(7,0, 0, MMC_CMDAT_R1|MMC_CMDAT_BUSY);
        if(resp)current_state = MMC_ST_STBY;

	return blkcnt;
}

/* TRAN_SPEED Multipliers mantissa is fixed point 10x book value */
u32 t_mantissa[] = {0,10,12,13, 15,20,25,30, 35,40,45,50, 55,60,70,80};      

/* Card transfer data rate exponents */
u32 tran_speed_exp[] = {10000,100000,1000000,1000000,0,0,0,0}; /* in HZ */

/* TACC multipliers */
u32 tacc_speed_exp[] = {1,10,100,1000,10000,100000,1000000,10000000}; /* in ns */

/****************************************************/
int
mmc_init(int verbose)
/****************************************************/
{
 	int retries, rc = -ENODEV;
	uchar *resp;
	u32 tmp1,tran=0,Tpwns,taccess=0;

	mmc_csd_t *csd = 0;

	card_type = CARD_MMC;

#ifdef CONFIG_LUBBOCK
	set_GPIO_mode( GPIO6_MMCCLK_MD );
	set_GPIO_mode( GPIO8_MMCCS0_MD );
#endif
#ifdef CONFIG_GUMSTIX
	set_GPIO_mode( GPIO6_MMCCLK_MD );
	set_GPIO_mode( GPIO8_MMCCS0_MD );
#endif
#ifdef CONFIG_FIONA
	set_GPIO_mode( GPIO6_MMCCLK_MD );

	// If we're on a P3 or later, MMC power is tied to GPIO 57.
	// We don't check against P1 as it's the same ID as P2
	if (FIONA_BOARD_P2 != get_fiona_board_revision()) {
		set_GPIO_mode( GPIO57_MMC_POWER_MD );
	}
#endif

	if (getenv(ENV_MBREAD_FLAG) != NULL) {
		mmc_use_mbread = 1;
		printf("(multi-block read support enabled)\n");
	}

	/* CKEN |= CKEN12_MMC;   enable MMC unit clock from core. Off only for power down (enabled at reset) */

	MMC_CLKRT  = MMC_CLKRT_0_3125MHZ;	/* 312 KHZ */
	MMC_RESTO  = MMC_RES_TO_MAX;
	MMC_SPI    = MMC_SPI_DISABLE;

	/* reset the current state status */
	current_state = MMC_ST_IDLE;

	/* Power On Reset Assumed for this test, the 1st command (55) prepends a 80 bit clock to reset card */
	retries = 5;
	resp = NULL;	/* let the loop work at least once */
	while(retries-- && !resp) {
	  /* djf */
	  /* mmc_cmd(ushort cmd, ushort argh, ushort argl, ushort cmdat) */
	  resp = mmc_cmd(MMC_CMD_RESET, 0, 0, MMC_CMDAT_INIT|MMC_CMDAT_BUSY|MMC_CMDAT_R1);
	  udelay(200000);
	} /* while */

	/* djf end */
	if(verbose) printf("Trying MMC...\n");
	retries = 5;
	rca_addr = MMC_DEFAULT_RCA;

	/* 2.7 - 3.3V */
	resp = mmc_cmd(MMC_CMD_SEND_OP_COND, 0x00ff, 0x8000, MMC_CMDAT_BUSY|MMC_CMDAT_R3);
	while (retries-- && resp && !(resp[4] & 0x80))
	{
		debug("     mmc_init>  resp 0x%02x 0x%02x\n", resp[0], resp[1]); /* was debug */
		udelay(200000); /* djf was 20000, now my JVC card detected 1st time */
		resp = mmc_cmd(MMC_CMD_SEND_OP_COND, 0x00ff, 0x8000, MMC_CMDAT_BUSY|MMC_CMDAT_R3);
	} /* while */
	if(resp)current_state = MMC_ST_READY;

	/* test for SD */
	if (current_state != MMC_ST_READY) {
		if(verbose) printf("Trying SD...\n");

		retries = 5;
		resp = NULL;    /* let the loop work at least once */
		while(retries-- && !resp) {
			resp = mmc_cmd(MMC_CMD_RESET, 0, 0, MMC_CMDAT_INIT|MMC_CMDAT_BUSY|MMC_CMDAT_R1);
			udelay(200000);
		} /* while */

		/* test for SDHC */
		retries = 5;
		resp = NULL;
		while (retries-- && !resp) {
			resp = mmc_cmd(MMC_CMD_SD_SEND_IF_COND, 0, 0x100 | MMC_SD_IF_COND_TEST, MMC_CMDAT_INIT|MMC_CMDAT_BUSY|MMC_CMDAT_R1); 
			udelay(50000);
		}

		/* if found, set the card type and tell the card controller
		   that we support SDHC block addressing */
		int ocrf = 0;
		if (resp && resp[1] == MMC_SD_IF_COND_TEST) {
			card_type = CARD_SD_2;

			ocrf = MMC_SD_IF_OCR_BLOCKADDR_EN;
		}

		retries = 10;
		while (retries--) {
			mmc_cmd(MMC_CMD_APP_CMD, 0, 0, MMC_CMDAT_BUSY|MMC_CMDAT_R1);
			resp = mmc_cmd(MMC_CMD_SD_APP_OP_COND, 0x0020 | ocrf, 0x0000, MMC_CMDAT_BUSY|MMC_CMDAT_R3);
			if (!resp) {
				break;
			}
			udelay(200000);
		}

		if (!resp) {
			if (card_type != CARD_SD_2) {
				card_type = CARD_SD_1;
			}

			rca_addr = SD_DEFAULT_RCA;
		}

		current_state = retries ? MMC_ST_READY : MMC_ST_IDLE;
	}

	if(current_state == MMC_ST_READY)
	{
		/* try to get card id both sd & mmc */
		resp = mmc_cmd(MMC_CMD_ALL_SEND_CID, 0, 0, MMC_CMDAT_BUSY|MMC_CMDAT_R2); /* djf MMC_CMDAT_BUSY */
		if (resp)
		{
			current_state = MMC_ST_IDENT; /* ident */

			if (card_type != CARD_MMC) {
				sd_cid_t *cid = (sd_cid_t *)resp;
				uchar year = cid->yearlo | cid->yearhi << 4;

				#define FIX_ZERO_C(n) ((n) ? (n) : ' ')
				sprintf(mmc_dev.product, "%c%c%c%c%c", 
					FIX_ZERO_C(cid->name[4]), FIX_ZERO_C(cid->name[3]), FIX_ZERO_C(cid->name[2]),
					FIX_ZERO_C(cid->name[1]), FIX_ZERO_C(cid->name[0]));

				if (verbose)
				{
					printf("\nSD card found:\n"
						   "  Manufacturer ID = 0x%02X\n"
						   "  OEM ID = \"%c%c\"\n"
						   "  HW Revision = 0x%02X\n"
						   "  Product Name = \"%s\"\n"
						   "  Serial Number = 0x%02X%02X%02X\n"
						   "  Date = %02d/%d\n",
						cid->mid,
						FIX_ZERO_C(cid->id[1]), FIX_ZERO_C(cid->id[0]),
						cid->fwrev,
						mmc_dev.product,
						cid->sn[2], cid->sn[1], cid->sn[0],
						cid->month, 2000 + year);
				} /* verbose */

				sprintf(mmc_dev.vendor,"Man %02x%02x%02x Snr %02x%02x%02x",
					cid->id[0], cid->id[1], cid->mid, cid->sn[0], cid->sn[1], cid->sn[2]);
				sprintf(mmc_dev.revision, "FW%02x", cid->fwrev);

				if (cid->mid == MID_TOSHIBA && card_type == CARD_SD_1) {
					/* force disable of multi-block reads for Toshiba SD 1.x cards */
					mmc_use_mbread = 0;
				}

			} else {
				mmc_cid_t *cid = (mmc_cid_t *)resp;
				u32 mid = cid->id[2] << 16 | cid->id[1] << 8 | cid->id[0];

				if (verbose)
				{
					printf("\nMMC card found:\n"
						   "  Manufacturer ID = 0x%06X\n"
						   "  HW/FW Revision = 0x%02X/0x%02X\n"
						   "  Product Name = \"%s\"\n"
						   "  Serial Number = 0x%02x%02x%02x\n"
						   "  Date = %02d/%d\n",
						mid,
						cid->hwrev, cid->fwrev,
						cid->name,
						cid->sn[2], cid->sn[1], cid->sn[0],
						cid->month, 1997 + cid->year);

					cid->hwrev = cid->fwrev = 0;	/* null terminate string */
				} /* verbose */

				sprintf(mmc_dev.vendor,"Man %06x Snr %02x%02x%02x",
					mid, cid->sn[0], cid->sn[1], cid->sn[2]);
				sprintf(mmc_dev.product,"%s", cid->name);
				sprintf(mmc_dev.revision,"HW%02x FW%02x", cid->hwrev, cid->fwrev);
			}
		} /* resp */ 
	} /* current_state == MMC_ST_READY */

	if(current_state == MMC_ST_IDENT) { /* ident */
		/* MMC You tell the card the address to use */
		resp = mmc_cmd(MMC_CMD_SET_RCA, rca_addr, 0, MMC_CMDAT_R1);
		if (resp) {
			current_state = MMC_ST_STBY;

			/* recompute rca_addr if SD */
			if (card_type != CARD_MMC) {
				rca_addr = (resp[4] << 8) | resp[3];
			}
		}

	} /* current_state == MMC_ST_READY */

	if (current_state == MMC_ST_STBY) { /* standby state */

		/* GET CSD */
		resp = mmc_cmd(MMC_CMD_SEND_CSD, rca_addr, 0, MMC_CMDAT_R2|MMC_CMDAT_BUSY);
		if (resp)
		{
			mmc_csd2_t *csd2;
			unsigned long total_size = 0;

			csd = (mmc_csd_t *)resp; /* djf */
			/* printf("sizeof csd = %d\n",sizeof(mmc_csd_t)); */
			/* memcpy(&mmc_csd, csd ,sizeof(mmc_csd_t)); */
			rc = 0;
			mmc_ready = 1;

			/* only CSD version 1 cards are SD 2.0; some card controllers support
			   the SD 2.0 initialization sequence, but are really SD 1.0 cards */ 
			if (card_type == CARD_SD_2 && csd->csd_structure == 0) {
				card_type = CARD_SD_1;
			}

			if (card_type != CARD_SD_2) {
				/* MMC, SD 1.x */ 
				taccess = (t_mantissa[((csd->taac>>3) & 0xf)] * tacc_speed_exp[(csd->taac & 7)])/10;
				tran = t_mantissa[((csd->tran_speed>>3) & 0xf)] * tran_speed_exp[(csd->tran_speed & 7)];

				printf("\nCSD_STRUCTURE=%x, MMC_PROT=%x\n",csd->csd_structure,csd->spec_vers);
				printf("  TAAC=%x (%dns), NSAC=%x\n",csd->taac,taccess,csd->nsac);
				printf("  TRAN=%x (%dbps)\n",csd->tran_speed,tran);
				printf("  CCC=%x, READ_BL_LEN=%x (%d)\n",csd->ccc,csd->read_bl_len,(1<<csd->read_bl_len));
				printf("  C_SIZE=%x, C_SIZE_MULT=%d (%d)\n",csd->c_size, csd->c_size_mult1,
				        (4<<csd->c_size_mult1));

				mmc_dev.lba = csd->c_size * (4 << csd->c_size_mult1);

				total_size = ((1<<csd->read_bl_len) * mmc_dev.lba) / 1024;

			} else {
				/* SD 2.0 */
				csd2 = (mmc_csd2_t *)resp;

				if (csd2->csd_structure != 1) {
					printf("\nWARNING: SD 2.0 card detected, CSD structure version is not 01\n");
				}

				printf("\nCSD_STRUCTURE=%x (%s), MMC_PROT=%x\n",csd2->csd_structure,csd2->csd_structure == 1 ? "SD 2.0" : "???",csd2->spec_vers);
				printf("  CCC=%x, READ_BL_LEN=%x (%d)\n",csd2->ccc,csd->read_bl_len,(1<<csd2->read_bl_len));
				printf("  C_SIZE=%x\n",csd2->c_size);

				/* force use of block reads for SDHC */
				mmc_use_mbread = 1;

				mmc_dev.lba = csd2->c_size * 1024;

				total_size = (1<<csd2->read_bl_len) * csd2->c_size ;

			}

			printf("\nTOTAL %s SIZE = %dMB\n", card_type == CARD_MMC ? "MMC" : "SD", total_size / 1024);

			/* Correct the sizes to tell fat fs */
			
			/* use partial block reads, so fix the size at MMC_BLOCK_SIZE */
			/* instead of supporting the native read_bl_len size */
			/* mmc_block_size = (1<<csd->read_bl_len); */
			mmc_block_size = MMC_BLOCK_SIZE;

		} /* if resp */
	
	}	/* current_state == MMC_ST_STBY */

	/*
		DO NOT OVER-WRITE resp[] at this point. It will corrupt the csd-> which
		points to the same place. No more cmm_cmd (or create you own buffered copy)
	*/

	MMC_STRPCL = MMC_STRPCL_STOP_CLK;

	if (card_type == CARD_SD_2) {
		MMC_CLKRT = 0;
		MMC_RDTO = 0xFFFF;

	} else {
		if(tran < 625000) {
			MMC_CLKRT = 6;	/* 312.5 KHZ Minimum speed */
			Tpwns = 3200;	/* Clock period */
			}
		else if(tran < 1250000) {
			MMC_CLKRT = 5;	/* 625 KHZ */
			Tpwns = 1600;
		}
		else if(tran < 2500000) {
			MMC_CLKRT = 4;	/* 1.25MHZ */
			Tpwns = 800; 
		}
		else if(tran < 5000000) {
			MMC_CLKRT = 3;	/* 2.5 MHZ */
			Tpwns = 400;
		}
		else if(tran < 10000000) {
			MMC_CLKRT = 2;	/* 5 MHZ */
			Tpwns = 200;
		}
		else if(tran < 20000000) {
			MMC_CLKRT = 1;	/* 10 MHZ */
			Tpwns = 100;
		}
		else {
			MMC_CLKRT = 0;	/* 20 MHZ */
			Tpwns = 50;
		}
	
		/* This is where I require csd-> to be intact */
		tmp1 = taccess / 50;				/* Number of clocks to = ns of rd access time */
		tmp1++;						/* Round up */
		tmp1 += ((Tpwns * 100 * csd->nsac) / 50);	/* Additional MMC RATE clocks of access time */
		
		/* The following would be (tmp1>>8), but I use a 4X safety factor */
		MMC_RDTO = (tmp1 >> 6);			/* Timeout period in 20MHZ clocks (50ns) */
		//printf("  MMC_RDTO = %d\n",(tmp1 >> 6));
	
	}

	/* fill in device description */
	mmc_dev.if_type = IF_TYPE_MMC;
	mmc_dev.dev = 1;
	mmc_dev.lun = 1;
	mmc_dev.blksz = mmc_block_size;		/* I dont think this is used */
	mmc_dev.part_type = PART_TYPE_DOS;
	mmc_dev.type = DEV_TYPE_UNKNOWN;
	mmc_dev.removable = 0;
	mmc_dev.block_read = mmc_bread;

	fat_register_device(&mmc_dev,1); /* partitions start counting with 1 */

    printf("\n");

	return rc;
}

int
mmc_ident(block_dev_desc_t *dev)
{
	return 0;
}

int
mmc2info(ulong addr)
{
	/* FIXME hard codes to 32 MB device */
	if (addr >= CFG_MMC_BASE && addr < CFG_MMC_BASE + 0x02000000)
	{
		return 1;
	}
	return 0;
}

#endif
