#ifndef __FIONA_IOC_H__
#define __FIONA_IOC_H__

#define IOC_KEY_PACKET_COMPLETE_OSCR_TIMEOUT    (1*CFG_HZ/4)    // 0.25 Seconds
#define IOC_KEY_WAIT_OSCR_TOTAL_TIMEOUT         (5*CFG_HZ)      // 5.00 Seconds
#define IOC_KEY_WAIT_OSCR_TIMEOUT               (1*CFG_HZ)      // 1.00 Second

#define IOC_KEY_WAIT_STILL_DOWN                 (2*CFG_HZ)      // 2.00 Seconds

#define OP_AMP_OFFSET_CALCULATION_SAMPLE_COUNT  5
#define MAMP_DRAW_WHILE_IDLE                    200     // Measured coming out of IOC reseting board
#define MIN_MAMP_DRAW_FOR_OP_AMP_OFFSET_CALC    180
#define MAX_MAMP_DRAW_FOR_OP_AMP_OFFSET_CALC    220

#define UNINITIALIZED_OP_AMP_OFFSET             0xFFFF  // Op Amp Offset not yet initialized, reboot board first
#define INITIALIZE_OP_AMP_OFFSET_STAGE_1        0xFFFE  // Op Amp Offset not yet initialized, measure now...
#define RESISTANCE_CALCULATION_SAMPLE_COUNT     10
#define RESISTANCE_DELTA_SAMPLE_COUNT           2
#define MV_CALCULATION_SAMPLE_COUNT             5
#define MAX_AMP_DIFFERNCE_FOR_RESISTANCE_CALC   1
#define IOC_RESPONSE_RETRIES                    5

#define IOC_KEY_RELEASE_BIT                     0x80
#define IOC_KEY_MASK_RAW                        0xFF

#define MAX_IOC_RX_PACKET_SIZE              8           // Max bytes in ioc rx packet
#define MAX_IOC_TX_PACKET_SIZE              32          // Max bytes in ioc rx packet

/****************************************
 *
 * IOC Command related constants
 *
 ****************************************/

// Reference Voltage Correction Related Constants
#define VOLTAGE_REFERENCE_BASE                  1000    // mv reference used for vref correction
#define MIN_VOLTAGE_REFERENCE_VALUE             975     // 2.05/2.1 = 976/1000
#define MAX_VOLTAGE_REFERENCE_VALUE             1025    // 2.15/2.1 = 1024/1000


// Return packet header types / bytes
#define IOC_SUCCESS_RESPONSE        0xFA
#define IOC_FAILURE_RESPONSE        0xFB
#define IOC_DISABLED_RESPONSE       0x00
#define IOC_ENABLED_RESPONSE        0x01

// Battery Status Responses
#define IOC_BATTERY_DISCHARGE_STATUS_RESPONSE   0x00
#define IOC_BATTERY_CHARGE_STATUS_RESPONSE      0x01
#define IOC_BATTERY_FULL_STATUS_RESPONSE        0x02
#define IOC_NO_BATTERY_STATUS_RESPONSE          0x03

#define IOC_TWO_BYTE_RESPONSE           0xE0
#define IOC_BATTERY_DISCHARGE_RESPONSE  0xE1
#define IOC_BATTERY_CHARGE_RESPONSE     0xE2
#define IOC_CRUSH_RESPONSE              0xE3
#define IOC_NO_BATTERY_RESPONSE         0xE4
#define IOC_FOUR_BYTE_RESPONSE          0xE7

#define IOC_AC_IN_RESPONSE              0xE5
#define IOC_AC_OUT_RESPONSE             0xE6

#define IOC_KEY_SCROLL_RESPONSE         0xF0
#define IOC_BATTERY_CAPACITY_RESPONSE   0xF1
#define IOC_BATTERY_LOW_RESPONSE        0xF2
#define IOC_ECHO_RESPONSE               0xFE

#define MAX_BYTES_IOC_RESPONSE          3
#define MAX_COMMAND_BYTES               32   // Size of internal IOC buffer
#define MAX_BYTES_IOC_COMMAND           MAX_COMMAND_BYTES
#define MIN_VALID_IOC_VERSION           0x07    // Minimum version supporting this protocol
#define MAX_VALID_IOC_VERSION           0xFE    // Maximum version supporting this protocol

#define IOC_ONE_KEY_WAKE_VERSION        26  // Added in 26
#define IOC_DUAL_KEY_WAKE_VERSION       29  // Added in 29

#define IOC_QUERY_CURRENT_DRAW                  0x34
#define IOC_QUERY_CURRENT_DRAW_RESPONSE_BYTES   2
#define IOC_QUERY_CURRENT_DRAW_RESPONSE_OFFSET  1

// Command and Response Byte Counts
#define IOC_CMD_ENABLE_BATTERY_LEVEL_REPORTING_CMD_BYTES        1
#define IOC_CMD_ENABLE_BATTERY_LEVEL_REPORTING_RESPONSE_BYTES   1
#define IOC_CMD_DISABLE_BATTERY_LEVEL_REPORTING_CMD_BYTES       1
#define IOC_CMD_DISABLE_BATTERY_LEVEL_REPORTING_RESPONSE_BYTES  1
#define IOC_CMD_BATTERY_LEVEL_EXPECTED_RESPONSE_BYTE            (0xFA)
#define IOC_QUERY_BATTERY_LEVEL_RESPONSE_BYTES                  2

#define IOC_CMD_ENABLE_BATTERY_LOW_WARNING_CMD_BYTES            2
#define IOC_CMD_ENABLE_BATTERY_LOW_WARNING_RESPONSE_BYTES       1
#define IOC_CMD_DISABLE_BATTERY_LOW_WARNING_CMD_BYTES           1
#define IOC_CMD_DISABLE_BATTERY_LOW_WARNING_RESPONSE_BYTES      1
#define IOC_CMD_BATTERY_LOW_EXPECTED_RESPONSE_BYTE              (0xFA)

#define IOC_CMD_ENABLE_PNLCD_CMD_BYTES                          1
#define IOC_CMD_ENABLE_PNLCD_RESPONSE_BYTES                     1
#define IOC_CMD_DISABLE_PNLCD_CMD_BYTES                         1
#define IOC_CMD_DISABLE_PNLCD_RESPONSE_BYTES                    1

#define IOC_CMD_PNLCD_LIGHT_RESPONSE_BYTES                      1

#define IOC_CMD_SET_IOC_TIMEOUT_CMD_BYTES                       2
#define IOC_CMD_SET_IOC_TIMEOUT_RESPONSE_BYTES                  1

#define IOC_CMD_SET_KEY_REPEAT_TIMEOUT_RESPONSE_BYTES           2


#define IOC_CMD_KEYFUNC_MASK_CMD_BYTES                          2
#define IOC_CMD_KEYFUNC_MASK_RESPONSE_BYTES                     1
#define IOC_CMD_KEYFUNC_UNMASK_CMD_BYTES                        2
#define IOC_CMD_KEYFUNC_UNMASK_RESPONSE_BYTES                   1

#define IOC_CMD_ENABLE_POWER_SAVINGS_CMD_BYTES                  1
#define IOC_CMD_ENABLE_POWER_SAVINGS_RESPONSE_BYTES             1
#define IOC_CMD_DISABLE_POWER_SAVINGS_CMD_BYTES                 6
#define IOC_CMD_DISABLE_POWER_SAVINGS_RESPONSE_BYTES            0

#define IOC_CMD_ENABLE_SCROLLWHEEL_CMD_BYTES                    1
#define IOC_CMD_ENABLE_SCROLLWHEEL_RESPONSE_BYTES               1
#define IOC_CMD_DISABLE_SCROLLWHEEL_CMD_BYTES                   1
#define IOC_CMD_DISABLE_SCROLLWHEEL_RESPONSE_BYTES              1

#define IOC_CMD_ENABLE_BUTTONS_CMD_BYTES                        1
#define IOC_CMD_ENABLE_BUTTONS_RESPONSE_BYTES                   1
#define IOC_CMD_DISABLE_BUTTONS_CMD_BYTES                       1
#define IOC_CMD_DISABLE_BUTTONS_RESPONSE_BYTES                  1

#define IOC_CMD_ENABLE_KEYBOARD_CMD_BYTES                       1
#define IOC_CMD_ENABLE_KEYBOARD_RESPONSE_BYTES                  1
#define IOC_CMD_DISABLE_KEYBOARD_CMD_BYTES                      1
#define IOC_CMD_DISABLE_KEYBOARD_RESPONSE_BYTES                 1

#define IOC_CMD_ENABLE_IOC_CMD_BYTES                            1
#define IOC_CMD_ENABLE_IOC_RESPONSE_BYTES                       1
#define IOC_CMD_DISABLE_IOC_CMD_BYTES                           1
#define IOC_CMD_DISABLE_IOC_RESPONSE_BYTES                      1
#define IOC_CMD_ENABLE_ALL_KEY_WAKE_CMD_BYTES                   1
#define IOC_CMD_ENABLE_ALL_KEY_WAKE_RESPONSE_BYTES              1
#define IOC_CMD_DISABLE_ALL_KEY_WAKE_CMD_BYTES                  1
#define IOC_CMD_DISABLE_ALL_KEY_WAKE_RESPONSE_BYTES             1
#define IOC_CMD_SET_ONE_KEY_WAKE_CMD_BYTES                      2
#define IOC_CMD_SET_ONE_KEY_WAKE_RESPONSE_BYTES                 1
#define IOC_CMD_ENABLE_ONE_KEY_WAKE_CMD_BYTES                   1
#define IOC_CMD_ENABLE_ONE_KEY_WAKE_RESPONSE_BYTES              1
#define IOC_CMD_DISABLE_ONE_KEY_WAKE_CMD_BYTES                  1
#define IOC_CMD_DISABLE_ONE_KEY_WAKE_RESPONSE_BYTES             1
#define IOC_CMD_ENABLE_TWO_KEY_WAKE_CMD_BYTES                   1
#define IOC_CMD_ENABLE_TWO_KEY_WAKE_RESPONSE_BYTES              1
#define IOC_CMD_DISABLE_TWO_KEY_WAKE_CMD_BYTES                  1
#define IOC_CMD_DISABLE_TWO_KEY_WAKE_RESPONSE_BYTES             1
#define IOC_CMD_SET_TWO_KEY_WAKE_CMD_BYTES                      3
#define IOC_CMD_SET_TWO_KEY_WAKE_RESPONSE_BYTES                 1

// Query Command Constants
#define IOC_QUERY_BATTERY_LEVEL                                 0x30
#define IOC_QUERY_PNLCD_POWER_STATUS                            0x20
#define IOC_QUERY_IOC_ENABLE_STATUS                             0x21
#define IOC_QUERY_FW_VERSION                                    0x31
#define IOC_QUERY_BATTERY_CAPACITY                              0x32
#define IOC_QUERY_BATTERY_TEMPERATURE                           0x33
#define IOC_QUERY_KEY_REPEAT_TIMEOUT                            0x36
#define IOC_QUERY_CHARGE_OR_DISCHARGE                           0x22
#define IOC_QUERY_CURRENT_DRAW                                  0x34
#define IOC_QUERY_KEY_STATUS                                    0x19
#define IOC_QUERY_KEY_MASK_STATUS                               0x1A
#define IOC_QUERY_KEYBOARD_ENABLE_STATUS                        0x23
#define IOC_QUERY_BUTTONS_ENABLE_STATUS                         0x24
#define IOC_QUERY_SCROLLWHEEL_ENABLE_STATUS                     0x26
#define IOC_QUERY_POWER_SAVING_STATUS                           0x27

#define IOC_QUERY_BATTERY_TEMPERATURE_RESPONSE_BYTES            2
#define IOC_QUERY_IOC_WAKE_SOURCE_RESPONSE_BYTES                2
#define IOC_QUERY_IOC_WAKE_SOURCE_CMD_BYTES                     1

// Version of IOC firmware that implements battery_info command
#define IOC_QUERY_BATTERY_INFO_CMD_VERSION                      49
#define IOC_QUERY_BATTERY_INFO                                  0x35
#define IOC_QUERY_BATTERY_INFO_CMD_BYTES                        1
#define IOC_QUERY_BATTERY_INFO_RESPONSE_PACKET_TYPE             IOC_FOUR_BYTE_RESPONSE
#define IOC_QUERY_BATTERY_INFO_RESPONSE_BYTES                   5
#define IOC_QUERY_BATTERY_INFO_VOLT_RESPONSE_OFFSET             1
#define IOC_QUERY_BATTERY_INFO_CHARGE_RESPONSE_OFFSET           2
#define IOC_QUERY_BATTERY_INFO_CHARGING_RESPONSE                0xE2
#define IOC_QUERY_BATTERY_INFO_DISCHARGING_RESPONSE             0xE1
#define IOC_QUERY_BATTERY_INFO_AMP_RESPONSE_OFFSET              3
#define IOC_QUERY_BATTERY_INFO_TEMP_RESPONSE_OFFSET             4

/*
 * IOC_QUERY_IOC_WAKE_SOURCE call will return 0xE0 0xNN,
 *  where 0xNN is one of the following wake sources 0-3:
 */
#define IOC_QUERY_IOC_WAKE_SOURCE                               0x3C
#define IOC_WOKEN_BY_AC_ADAPTER                                 0x00
#define IOC_WOKEN_BY_LOW_BATTERY                                0x01
#define IOC_WOKEN_BY_UART                                       0x02
#define IOC_WOKEN_BY_KEY_PRESS                                  0x03

// Control Command Constants
#define IOC_CMD_DISABLE_POWER_SAVINGS                           0x00
#define IOC_CMD_ENABLE_POWER_SAVINGS                            0x01
#define IOC_CMD_DISABLE_KEYBOARD                                0x02
#define IOC_CMD_ENABLE_KEYBOARD                                 0x03
#define IOC_CMD_DISABLE_BUTTONS                                 0x04
#define IOC_CMD_ENABLE_BUTTONS                                  0x05
#define IOC_CMD_DISABLE_SCROLLWHEEL                             0x08
#define IOC_CMD_ENABLE_SCROLLWHEEL                              0x09
#define IOC_CMD_DISABLE_PNLCD                                   0x0A
#define IOC_CMD_ENABLE_PNLCD                                    0x0B
#define IOC_CMD_DISABLE_IOC                                     0x0C
#define IOC_CMD_ENABLE_IOC                                      0x0D
#define IOC_CMD_DISABLE_BATTERY_CAPACITY_REPORTING              0x0E
#define IOC_CMD_ENABLE_BATTERY_CAPACITY_REPORTING               0x0F
#define IOC_CMD_DISABLE_BATTERY_LOW_REPORTING                   0x10
#define IOC_CMD_ENABLE_BATTERY_LOW_REPORTING                    0x11
#define IOC_CMD_RESET_FIONA                                     0x16
#define IOC_CMD_SET_KEY_REPEAT_TIMEOUT                          0x37
#define IOC_CMD_ENABLE_ALL_KEY_WAKE                             0x38
#define IOC_CMD_DISABLE_ALL_KEY_WAKE                            0x39
#define IOC_CMD_ENABLE_ONE_KEY_WAKE                             0x3A
#define IOC_CMD_SET_TWO_KEY_WAKE                                0x3B
#define IOC_CMD_ENABLE_TWO_KEY_WAKE                             0x3A
#define IOC_CMD_DISABLE_TWO_KEY_WAKE                            IOC_CMD_DISABLE_ALL_KEY_WAKE
#define IOC_CMD_MASK_KEY_FUNCTION                               0x17
#define IOC_CMD_UNMASK_KEY_FUNCTION                             0x18
#define IOC_CMD_ECHO                                            0xFE
#define IOC_CMD_SW_RESET                                        0xFF

#define IOC_CMD_PNLCD_SEQUENTIAL_ON                             0x40
#define IOC_CMD_PNLCD_SEQUENTIAL_OFF                            0x41
#define IOC_CMD_PNLCD_VERTICAL_SEQUENTIAL_ON                    0x42
#define IOC_CMD_PNLCD_VERTICAL_SEQUENTIAL_OFF                   0x43
#define IOC_CMD_PNLCD_MISC_ON                                   0x44
#define IOC_CMD_PNLCD_MISC_OFF                                  0x45
#define IOC_CMD_PNLCD_MISC_PAIR_ON                              0x46
#define IOC_CMD_PNLCD_MISC_PAIR_OFF                             0x47
#define IOC_CMD_PNLCD_MISC_VERTICAL_PAIR_ON                     0x48
#define IOC_CMD_PNLCD_MISC_VERTICAL_PAIR_OFF                    0x49

#define IOC_CMD_PROGRAMMER_DATA_LENGTH                          0x60
#define IOC_CMD_PROGRAMMER_SEND_DATA                            0x61
#define IOC_CMD_PROGRAMMER_SEND_COMPLETE                        0x62
#define IOC_CMD_PROGRAMMER_READ                                 0x63
#define IOC_CMD_PROGRAMMER_ROM_DUMP                             0x64
#define IOC_CMD_PROGRAMMER_START_PROGRAMMING                    0x65


typedef enum IOC_STATE {
    IOC_STATE_FIND_PACKET,
    IOC_STATE_KEY_HEADER_FOUND,
    IOC_STATE_BURN_PACKET,
    IOC_STATE_DONE
}IOC_STATE;

extern int dbg_bt_tstc(void);
extern int dbg_bt_getc(void);
extern int dbg_bt_putc(unsigned int c);

#endif // __FIONA_IOC_H__
